function [error_train, error_val] = learningCurve(X, y, Xval, yval, lambda)

%   LEARNINGCURVE Generates the train and cross validation set errors 
%   needed to plot a learning curve
%   [error_train, error_val] = LEARNINGCURVE(X, y, Xval, yval, lambda) 
%   returns the train and cross validation set errors for a learning curve. 
%   In particular, it returns two vectors of the same length :
%   error_train(i) contains the training error for i examples 
%   and similarly for error_val(i)

m = size(X, 1); % dim(X) = (m , n+1)
error_train = zeros(m, 1); % training error
error_val   = zeros(m, 1); % cross validation error
for i = 1:m
    % Determine le meilleur vecteur Theta 
    % en utilisant les i premiers exemples du training set
    theta = trainLinearReg(X(1:i,:), y(1:i), lambda);
    % Jamais de régularisation pour training error & cross validation error
    % Calcul de l'erreur sur les i premiers exemples du training set
    [error_train(i),~] = linearRegCostFunction(X(1:i,:), y(1:i), theta, 0);
    % Calcul de l'erreur sur les tous les exemples du cross validation set
    [error_val(i),~]= linearRegCostFunction(Xval, yval, theta, 0);
end

end
