function [J, grad] = linearRegCostFunction(X, y, theta, lambda)

%   LINEARREGCOSTFUNCTION Compute cost and gradient for 
%   regularized linear regression with multiple variables
%   [J, grad] = LINEARREGCOSTFUNCTION(X, y, theta, lambda) computes the 
%   cost of using theta as the parameter for linear regression to fit the 
%   data points in X and y. Returns the cost in J and the gradient in grad

m = size(X,1); % dim(X) = (m , n+1)
HthetaX_y = X*theta - y; % (m , 1)

J = (1/2/m)*(HthetaX_y'*HthetaX_y + lambda*(theta(2:end)'*theta(2:end)));

grad = (X'*HthetaX_y)/m; % (n+1 , 1)
grad(2:end) = grad(2:end) + (lambda/m)*theta(2:end);

end
